#if !defined(_BTPAY_SDK_H_)
#define _BTPAY_SDK_H_

////////////////////////////////////////
//SDK v1.0 IDTECH

#ifndef __BTPAY__
#define __BTPAY_API __declspec(dllimport)
#else
#define __BTPAY_API __declspec(dllexport)
#endif
#define BYTE unsigned char

#define INTERFACE_USBHID		0
#define INTERFACE_BLUETOOTH		1

typedef enum{
  RET_DO_SUCCESS=0,
  RET_ERR_CMD_FAIL=1,
  RET_ERR_TIME_OUT,
  RET_ERR_NOT_OPEN,
  RET_ERR_INVALID_PARAMETER,
  RET_SDK_BUZY_MSR,
  RET_SDK_BUZY_PINPAD,
  RET_ERR_OTHER
}ResultType;


//main APIs
extern "C" 
{
typedef enum{
  POWER_ON_OPTION_IFS_FLAG=1,
  POWER_ON_OPTION_EXPLICIT_PPS_FLAG=2,
  POWER_ON_OPTION_AUTO_PPS_FLAG=64,
  POWER_ON_OPTION_IFS_RESPONSE_CHECK_FLAG=128
}PowerOnOption;


//RSA key struct
#ifndef _R_STUCT
#define _R_STUCT

#define MAX_RSA_MODULUS_BITS 2048
#define MAX_RSA_MODULUS_LEN ((MAX_RSA_MODULUS_BITS + 7) / 8)
#define MAX_RSA_PRIME_BITS ((MAX_RSA_MODULUS_BITS + 1) / 2)
#define MAX_RSA_PRIME_LEN ((MAX_RSA_PRIME_BITS + 7) / 8)

// RSA public and private key.
typedef struct {
    unsigned short int bits;                     // length in bits of modulus
    unsigned char modulus[MAX_RSA_MODULUS_LEN];  // modulus
    unsigned char exponent[MAX_RSA_MODULUS_LEN]; // public exponent
} R_RSA_PUBLIC_KEY;

//private key include p, q, d mod (p-1), d mod(q-1), q(-1) mod p
typedef struct {
    unsigned short int bits;                     // length in bits of modulus
    unsigned char modulus[MAX_RSA_MODULUS_LEN];  // modulus
    unsigned char publicExponent[MAX_RSA_MODULUS_LEN];     // public exponent
    unsigned char exponent[MAX_RSA_MODULUS_LEN]; // private exponent
    unsigned char prime[2][MAX_RSA_PRIME_LEN];   // prime factors
    unsigned char primeExponent[2][MAX_RSA_PRIME_LEN];     // exponents for CRT
    unsigned char coefficient[MAX_RSA_PRIME_LEN];          // CRT coefficient
} R_RSA_PRIVATE_KEY;

#endif //_R_STUCT

typedef enum{
  TYPE_EVENT_MSR_CARD_DATA = 0,
  TYPE_EVENT_MSR_CANCEL_KEY = 1,
  TYPE_EVENT_MSR_BACKSPACE_KEY = 2,
  TYPE_EVENT_MSR_ENTER_KEY = 3,
  TYPE_EVENT_MSR_UNKNOWN = 4
}TYPE_EVENT_MSR;

typedef enum{
  TYPE_EVENT_PIN_INPUT_DATA = 0,
  TYPE_EVENT_PIN_CANCEL = 1,
  TYPE_EVENT_PIN_TIMEOUT = 2,
  TYPE_EVENT_UNKNOWN = 3
}TYPE_EVENT_PIN;


#define IN
#define OUT
#define IN_OUT

	//common part
	typedef void (WINAPI *BTPAY_CALLBACKF)(OUT int, OUT BYTE*, OUT int);

	__BTPAY_API BOOL WINAPI BTPay_SetCmdTimeOutDuration(int nTimeOutDuration);
	__BTPAY_API BOOL WINAPI BTPay_Open(IN int nInterfaceType, IN int nSerialPort);
	__BTPAY_API void WINAPI BTPay_Close();
	__BTPAY_API int WINAPI BTPay_comGetFirmwareVersion(OUT char* cFirm);
	__BTPAY_API int WINAPI BTPay_comGetSerialNumber(OUT char* cSN);
	__BTPAY_API int WINAPI BTPay_comSetSerialNumber(IN char* cSN);
	__BTPAY_API int WINAPI BTPay_comGetModelNumber(OUT char* cMN);
	__BTPAY_API int WINAPI BTPay_comEnterSleepMode();
	__BTPAY_API int WINAPI BTPay_comSetIdleTimeForSleep(IN int nSecond);
	__BTPAY_API int WINAPI BTPay_comGetDateTime(OUT char* cDateTime);
	__BTPAY_API int WINAPI BTPay_comSetDateTime(IN char* cDateTime);
	__BTPAY_API int WINAPI BTPay_comDisplayMessage(IN char *cLineText);
	__BTPAY_API int WINAPI BTPay_comBeeperControl(IN BYTE *cControlItems, IN int nItemsCount);
	__BTPAY_API int WINAPI BTPay_comRebootDevice();
	__BTPAY_API int WINAPI BTPay_comSetBluetoothAddress(IN BYTE* cAddr);
	__BTPAY_API int WINAPI BTPay_comDirectIO(IN_OUT BYTE *cData, IN_OUT int &dataLen);

	//smart card part
	__BTPAY_API int WINAPI BTPay_smartGetICCReaderStatus(OUT BYTE &cStatus);
	__BTPAY_API int WINAPI BTPay_smartPowerOn( IN BYTE *optionFlags, IN int flagsLen,
										IN BYTE *ppsBytes, IN int ppsLen,
										OUT BYTE *ATR, OUT int &atrLen);
	__BTPAY_API int WINAPI BTPay_smartPowerOff();
	__BTPAY_API int WINAPI BTPay_smartExchangeAPDU(IN_OUT BYTE *cAPDU, IN_OUT int &apduLen);
	
	//MSR part
	__BTPAY_API BOOL WINAPI BTPay_msrAddCallBack_MSR(IN BTPAY_CALLBACKF func);
	__BTPAY_API int WINAPI BTPay_msrEnableMSR(IN char *cLineText);
	__BTPAY_API int WINAPI BTPay_msrDisableMSR();
	__BTPAY_API int WINAPI BTPay_msrGetMSR_dataFormat(OUT char &cFormat);
	__BTPAY_API int WINAPI BTPay_msrSetMSR_dataFormat(IN char &cFormat);

	//PinPad part
	__BTPAY_API int WINAPI BTPay_pinGetPinPadStatus(OUT BYTE &cStatus);
	__BTPAY_API int WINAPI BTPay_pinGetPinLength(OUT int &nLenMin, OUT int &nLenMax);
	__BTPAY_API int WINAPI BTPay_pinSetPinLength(IN int nLenMin, IN int nLenMax);

	__BTPAY_API BOOL WINAPI BTPay_pinAddCallBack_EncryptedPin(IN BTPAY_CALLBACKF func);
	__BTPAY_API int WINAPI BTPay_pinGetEncryptedPin(IN BYTE keyType,
													IN char * Account,
													IN char *cLineText);

	__BTPAY_API BOOL WINAPI BTPay_pinAddCallBack_Numeric(IN BTPAY_CALLBACKF func);
	__BTPAY_API int WINAPI BTPay_pinGetNumeric (IN BYTE echo_flag,
												IN BYTE Max_len,
												IN BYTE Min_len,
												IN char * cLineText,
												IN R_RSA_PRIVATE_KEY *rsa_Key);

	__BTPAY_API BOOL WINAPI BTPay_pinAddCallBack_Amount(IN BTPAY_CALLBACKF func);
	__BTPAY_API int WINAPI BTPay_pinGetAmount ( IN BYTE Max_len,
												IN BYTE Min_len,
												IN char * cLineText,
												IN R_RSA_PRIVATE_KEY *rsa_Key);

	__BTPAY_API BOOL WINAPI BTPay_pinAddCallBack_Account(IN BTPAY_CALLBACKF func);
	__BTPAY_API int WINAPI BTPay_pinGetAccount( IN BYTE Max_len,
												IN BYTE Min_len,
												IN char * cLineText);

	__BTPAY_API BOOL WINAPI BTPay_pinAddCallBack_EncryptedData(IN BTPAY_CALLBACKF func);
	__BTPAY_API int WINAPI BTPay_pinGetEncryptedData (IN BYTE end_flag,
												IN BYTE Max_len,
												IN BYTE Min_len,
												IN char * cLineText,
												IN R_RSA_PRIVATE_KEY *rsa_Key);

	__BTPAY_API int WINAPI BTPay_pinCancelPin();

	__BTPAY_API BOOL WINAPI BTPay_comErrorCodeScript(IN int nCode, OUT char* cSpt);

} //extern "C" 

#endif //_BTPAY_SDK_H_